@@ -1,57 +1,49 @@
 # RDF::Trine::Iterator::Bindings
+# -------------
+# $Revision $
+# $Date $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
-RDF::Trine::Iterator::Bindings - Iterator class for bindings query results
-
-=head1 VERSION
-
-This document describes RDF::Trine::Iterator::Bindings version 0.999_02
+RDF::Trine::Iterator::Bindings - Stream (iterator) class for bindings query results.
 
 =head1 SYNOPSIS
 
- use RDF::Trine::Iterator::Bindings;
- 
- my $iterator = RDF::Trine::Iterator::Bindings->new( \&data, \@names );
- while (my $row = $iterator->next) {
-   # $row is a HASHref containing variable name -> RDF Term bindings
-   my @vars = keys %$row;
-   print $row->{ 'var' }->as_string;
- }
+    use RDF::Trine::Iterator;
+    
+    my $iterator = RDF::Trine::Iterator::Bindings->new( \&data, \@names );
+    while (my $row = $iterator->next) {
+    	my @vars	= keys %$row;
+    	# do something with @vars
+    }
 
 =head1 METHODS
 
-Beyond the methods documented below, this class inherits methods from the
-L<RDF::Trine::Iterator> class.
-
 =over 4
 
 =cut
 
 package RDF::Trine::Iterator::Bindings;
 
-use utf8;
 use strict;
 use warnings;
 no warnings 'redefine';
 use Data::Dumper;
 
 use JSON 2.0;
-use Text::Table;
-use Log::Log4perl;
-use Scalar::Util qw(blessed reftype);
+use Scalar::Util qw(reftype);
 use RDF::Trine::Iterator::Bindings::Materialized;
-use RDF::Trine::Serializer::Turtle;
 
 use RDF::Trine::Iterator qw(smap);
 use base qw(RDF::Trine::Iterator);
 
-use Carp qw(croak);
-
-our ($VERSION);
+our ($REVISION, $VERSION, $debug);
+use constant DEBUG	=> 0;
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug		= DEBUG;
+	$REVISION	= do { my $REV = (qw$Revision: 293 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
+	$VERSION	= '1.000';
 }
 
 =item C<new ( \@results, \@names, %args )>
@@ -59,22 +51,26 @@ BEGIN {
 =item C<new ( \&results, \@names, %args )>
 
 Returns a new SPARQL Result interator object. Results must be either
-a reference to an array containing results or a CODE reference that
+an reference to an array containing results or a CODE reference that
 acts as an iterator, returning successive items when called, and
 returning undef when the iterator is exhausted.
 
+$type should be one of: bindings, boolean, graph.
+
 =cut
 
 sub new {
 	my $class	= shift;
 	my $stream	= shift || sub { undef };
 	my $names	= shift || [];
+	Carp::confess unless (scalar(@_) % 2 == 0);
 	my %args	= @_;
 	
 	my $type	= 'bindings';
 	my $self	= $class->SUPER::new( $stream, $type, $names, %args );
 	
 	my $s 	= $args{ sorted_by } || [];
+	Carp::confess unless (reftype($s) eq 'ARRAY');
 	$self->{sorted_by}	= $s;
 	return $self;
 }
@@ -91,9 +87,6 @@ sub _new {
 =item C<< materialize >>
 
 Returns a materialized version of the current binding iterator.
-The materialization process will leave this iterator empty. The materialized
-iterator that is returned should be used for any future need for the iterator's
-data.
 
 =cut
 
@@ -145,9 +138,10 @@ sub join_streams {
 # 	my $bridge	= shift;
 	my %args	= @_;
 	
-# 	Carp::confess unless ($a->isa('RDF::Trine::Iterator::Bindings'));
-# 	Carp::confess unless ($b->isa('RDF::Trine::Iterator::Bindings'));
-	my $l		= Log::Log4perl->get_logger("rdf.trine.iterator.bindings");
+#	my $debug	= $args{debug};
+	
+	Carp::confess unless ($a->isa('RDF::Trine::Iterator::Bindings'));
+	Carp::confess unless ($b->isa('RDF::Trine::Iterator::Bindings'));
 	
 	my @join_sorted_by;
 	if (my $o = $args{ orderby }) {
@@ -155,126 +149,52 @@ sub join_streams {
 		my $a_sort		= join(',', $a->sorted_by);
 		my $b_sort		= join(',', $b->sorted_by);
 		
-		if ($l->is_debug) {
-			$l->debug('---------------------------');
-			$l->debug('REQUESTED SORT in JOIN: ' . Dumper($req_sort));
-			$l->debug('JOIN STREAM SORTED BY: ' . Dumper($a_sort));
-			$l->debug('JOIN STREAM SORTED BY: ' . Dumper($b_sort));
+		if (DEBUG) {
+			warn '---------------------------';
+			warn 'REQUESTED SORT in JOIN: ' . Dumper($req_sort);
+			warn 'JOIN STREAM SORTED BY: ' . Dumper($a_sort);
+			warn 'JOIN STREAM SORTED BY: ' . Dumper($b_sort);
 		}
 		my $actual_sort;
 		if (substr( $a_sort, 0, length($req_sort) ) eq $req_sort) {
-			$l->debug("first stream is already sorted. using it in the outer loop.");
+			warn "first stream is already sorted. using it in the outer loop.\n" if (DEBUG);
 		} elsif (substr( $b_sort, 0, length($req_sort) ) eq $req_sort) {
-			$l->debug("second stream is already sorted. using it in the outer loop.");
+			warn "second stream is already sorted. using it in the outer loop.\n" if (DEBUG);
 			($a,$b)	= ($b,$a);
 		} else {
 			my $a_common	= join('!', $a_sort, $req_sort);
 			my $b_common	= join('!', $b_sort, $req_sort);
 			if ($a_common =~ qr[^([^!]+)[^!]*!\1]) {	# shared prefix between $a_sort and $req_sort?
-				$l->debug("first stream is closely sorted ($1).");
+				warn "first stream is closely sorted ($1).\n" if (DEBUG);
 			} elsif ($b_common =~ qr[^([^!]+)[^!]*!\1]) {	# shared prefix between $b_sort and $req_sort?
-				$l->debug("second stream is closely sorted ($1).");
+				warn "second stream is closely sorted ($1).\n" if (DEBUG);
 				($a,$b)	= ($b,$a);
 			}
 		}
 		@join_sorted_by	= ($a->sorted_by, $b->sorted_by);
 	}
 	
-	my $stream	= $self->nested_loop_join( $a, $b, %args );
-	$l->debug("JOINED stream is sorted by: " . join(',', @join_sorted_by));
+	my $stream	= $self->SUPER::join_streams( $a, $b, %args );
+	warn "JOINED stream is sorted by: " . join(',', @join_sorted_by) . "\n" if (DEBUG);
 	$stream->{sorted_by}	= \@join_sorted_by;
 	return $stream;
 }
 
-=item C<< nested_loop_join ( $outer, $inner ) >>
+=item C<< next_result >>
 
-Performs a natural, nested loop join of the two streams, returning a new stream
-of joined results.
-
-Note that the values from the C<< $inner >> iterator are fully materialized for
-this join, and the results of the join are in the order of values from the
-C<< $outer >> iterator. This suggests that:
-
-* If sorting needs to be preserved, the C<< $outer >> iterator should be used to
-determine the result ordering.
-
-* If one iterator is much smaller than the other, it should likely be used as
-the C<< $inner >> iterator since materialization will require less total memory.
+Returns the next binding result.
 
 =cut
 
-sub nested_loop_join {
+sub next_result {
 	my $self	= shift;
-	my $astream	= shift;
-	my $bstream	= shift;
-#	my $bridge	= shift;
-	my %args	= @_;
-	
-# 	Carp::confess unless ($astream->isa('RDF::Trine::Iterator::Bindings'));
-# 	Carp::confess unless ($bstream->isa('RDF::Trine::Iterator::Bindings'));
-	my $l		= Log::Log4perl->get_logger("rdf.trine.iterator.bindings");
-	
-	my @names	= RDF::Trine::_uniq( map { $_->binding_names() } ($astream, $bstream) );
-	my $a		= $astream->project( @names );
-	my $b		= $bstream->project( @names );
-	
-	
-	my @data	= $b->get_all();
-	no warnings 'uninitialized';
-	
-	my $inner_index;
-	my $rowa;
-	my $need_new_a	= 1;
-	my $sub	= sub {
-		OUTER: while (1) {
-			if ($need_new_a) {
-				$l->debug("### fetching new outer tuple");
-				$rowa = $a->next;
-				$inner_index	= 0;
-				$need_new_a		= 0;
-			}
-			$l->debug("OUTER: " . Dumper($rowa));
-			return undef unless ($rowa);
-			LOOP: while ($inner_index <= $#data) {
-				my $rowb	= $data[ $inner_index++ ];
-				$l->debug("- INNER[ $inner_index ]: " . Dumper($rowb));
-				$l->debug("[--JOIN--] " . join(' ', map { my $row = $_; '{' . join(', ', map { join('=', $_, ($row->{$_}) ? $row->{$_}->as_string : '(undef)') } (keys %$row)) . '}' } ($rowa, $rowb)));
-				my %keysa	= map {$_=>1} (keys %$rowa);
-				my @shared	= grep { $keysa{ $_ } } (keys %$rowb);
-				foreach my $key (@shared) {
-					my $val_a	= $rowa->{ $key };
-					my $val_b	= $rowb->{ $key };
-					my $defined	= 0;
-					foreach my $n ($val_a, $val_b) {
-						$defined++ if (defined($n));
-					}
-					if ($defined == 2) {
-						my $equal	= $val_a->equal( $val_b );
-						unless ($equal) {
-							$l->debug("can't join because mismatch of $key (" . join(' <==> ', map {$_->as_string} ($val_a, $val_b)) . ")");
-							next LOOP;
-						}
-					}
-				}
-				
-				my $row	= { (map { $_ => $rowa->{$_} } grep { defined($rowa->{$_}) } keys %$rowa), (map { $_ => $rowb->{$_} } grep { defined($rowb->{$_}) } keys %$rowb) };
-				if ($l->is_debug) {
-					$l->debug("JOINED:");
-					foreach my $key (keys %$row) {
-						$l->debug("$key\t=> " . $row->{ $key }->as_string);
-					}
-				}
-				return $row;
-			}
-			$need_new_a	= 1;
-		}
-	};
-	
-	my $args	= $astream->_args;
-	return $astream->_new( $sub, 'bindings', \@names, %$args );
+	my $data	= $self->SUPER::next_result();
+	if (defined($data)) {
+		Carp::confess "not a HASH ref" unless (reftype($data) eq 'HASH');
+	}
+	return $data;
 }
 
-
 =item C<< sorted_by >>
 
 =cut
@@ -298,8 +218,7 @@ sub binding_value_by_name {
 	if (exists( $row->{ $name } )) {
 		return $row->{ $name };
 	} else {
-# 		warn "No variable named '$name' is present in query results.\n";
-		return;
+		warn "No variable named '$name' is present in query results.\n";
 	}
 }
 
@@ -365,8 +284,10 @@ Returns the number of variable bindings in the current result.
 sub bindings_count {
 	my $self	= shift;
 	my $names	= $self->{_names};
+	my $row		= ($self->open) ? $self->current : $self->next;
 	return scalar( @$names ) if (scalar(@$names));
-	return 0;
+	return 0 unless ref($row);
+	return scalar( @{ [ keys %$row ] } );
 }
 
 =item C<is_bindings>
@@ -405,14 +326,22 @@ sub as_json {
 	
 	my $data	= {
 					head	=> { vars => \@variables },
-					results	=> { ordered => $order, distinct => $dist, bindings => [] },
+					results	=> { ordered => $order, distinct => $dist },
 				};
 	my @bindings;
-	while (my $row = $self->next) {
-		my %row	= map { format_node_json($row->{$_}, $_) } (keys %$row);
+	while (!$self->finished) {
+		my %row;
+		for (my $i = 0; $i < $width; $i++) {
+			my $name		= $self->binding_name($i);
+			my $value		= $self->binding_value($i);
+			if (my ($k, $v) = format_node_json($value, $name)) {
+				$row{ $k }		= $v;
+			}
+		}
+		
 		push(@{ $data->{results}{bindings} }, \%row);
 		last if ($max_result_size and ++$count >= $max_result_size);
-	}
+	} continue { $self->next_result }
 	
 	return to_json( $data );
 }
@@ -426,95 +355,8 @@ Returns an XML serialization of the stream data.
 sub as_xml {
 	my $self			= shift;
 	my $max_result_size	= shift || 0;
-	my $string	= '';
-	open( my $fh, '>', \$string );
-	$self->print_xml( $fh, $max_result_size );
-	close($fh);
-	return $string;
-}
-
-=item C<as_string ( $max_size [, \$count] )>
-
-Returns a string table serialization of the stream data.
-
-=cut
-
-sub as_string {
-	my $self			= shift;
-	my $max_result_size	= shift || 0;
-	my $rescount		= shift;
-	my @names			= $self->binding_names;
-	my $headers			= \@names;
-	my @rows;
-	my $count	= 0;
-	while (my $row = $self->next) {
-		push(@rows, [ map { blessed($_) ? RDF::Trine::Serializer::Turtle->node_as_concise_string($_) : '' } @{ $row }{ @names } ]);
-		last if ($max_result_size and ++$count >= $max_result_size);
-	}
-	if (ref($rescount)) {
-		$$rescount	= scalar(@rows);
-	}
-	
-	my @rule			= qw(- +);
-	my @headers			= (\q"| ");
-	push(@headers, map { $_ => \q" | " } @$headers);
-	pop	@headers;
-	push @headers => (\q" |");
-	
-	if ('ARRAY' eq ref $rows[0]) {
-		if (@$headers == @{ $rows[0] }) {
-			my $table = Text::Table->new(@headers);
-			$table->rule(@rule);
-			$table->body_rule(@rule);
-			$table->load(@rows);
-		
-			return join('',
-					$table->rule(@rule),
-					$table->title,
-					$table->rule(@rule),
-					map({ $table->body($_) } 0 .. @rows),
-					$table->rule(@rule)
-				);
-		} else {
-			die("make_table() rows must be an AoA with rows being same size as headers");
-		}
-	} else {
-		return '';
-	}
-}
-
-=item C<< as_statements ( @names ) >>
-
-Returns a L<RDF::Trine::Iterator::Graph> with the statements of the stream.
-
-=cut
-
-sub as_statements {
-	my $self	= shift;
-	my @names	= @_;
-	my $sub		= sub {
-		my $row	= $self->next;
-		return undef unless (defined $row);
-		my @values	= @{ $row }{ @names };
-		my $statement	= (scalar(@values) == 3 or not(defined($values[3])))
-						? RDF::Trine::Statement->new( @values[ 0 .. 2 ] )
-						: RDF::Trine::Statement::Quad->new( @values );
-		return $statement;
-	};
-	return RDF::Trine::Iterator::Graph->new( $sub )
-}
-
-=item C<< print_xml ( $fh, $max_size ) >>
-
-Prints an XML serialization of the stream data to the filehandle $fh.
-
-=cut
-
-sub print_xml {
-	my $self			= shift;
-	my $fh				= shift;
-	my $max_result_size	= shift || 0;
 	my $width			= $self->bindings_count;
+# 	my $bridge			= $self->bridge;
 	
 	my @variables;
 	for (my $i=0; $i < $width; $i++) {
@@ -522,40 +364,33 @@ sub print_xml {
 		push(@variables, $name) if $name;
 	}
 	
-	no strict 'refs';
-	print {$fh} <<"END";
-<?xml version="1.0" encoding="utf-8"?>
-<sparql xmlns="http://www.w3.org/2005/sparql-results#">
+	my $count	= 0;
+	my $t	= join("\n\t", map { qq(<variable name="$_"/>) } @variables);
+	my $xml	= <<"END";
+<?xml version="1.0"?>
+<sparql xmlns="http://www.w3.org/2001/sw/DataAccess/rf1/result2">
 <head>
-END
-	
-	my $t	= join("\n", map { qq(\t<variable name="$_"/>) } @variables);
-	
-	if ($t) {
-		print {$fh} "${t}\n";
-	}
-	
-	print {$fh} <<"END";
+	${t}
 </head>
 <results>
 END
-	
-	my $count	= 0;
-	while (my $row = $self->next) {
+	while (!$self->finished) {
 		my @row;
-		print {$fh} "\t\t<result>\n";
+		$xml	.= "\t\t<result>\n";
 		for (my $i = 0; $i < $width; $i++) {
-			my $name	= $self->binding_name($i);
-			my $value	= $row->{ $name };
-			print {$fh} "\t\t\t" . $self->format_node_xml($value, $name) . "\n";
+			my $name		= $self->binding_name($i);
+			my $value		= $self->binding_value($i);
+			$xml	.= "\t\t\t" . $self->format_node_xml($value, $name) . "\n";
 		}
-		print {$fh} "\t\t</result>\n";
+		$xml	.= "\t\t</result>\n";
 		
 		last if ($max_result_size and ++$count >= $max_result_size);
-	}
-	
-	print {$fh} "</results>\n";
-	print {$fh} "</sparql>\n";
+	} continue { $self->next_result }
+	$xml	.= <<"EOT";
+</results>
+</sparql>
+EOT
+	return $xml;
 }
 
 =begin private
@@ -578,13 +413,13 @@ sub format_node_json ($$$) {
 	
 	if(!defined $node) {
 		return;
-	} elsif ($node->isa('RDF::Trine::Node::Resource')) {
+	} elsif ($node->is_resource) {
 		$node_label	= $node->uri_value;
 		return $name => { type => 'uri', value => $node_label };
-	} elsif ($node->isa('RDF::Trine::Node::Literal')) {
+	} elsif ($node->is_literal) {
 		$node_label	= $node->literal_value;
 		return $name => { type => 'literal', value => $node_label };
-	} elsif ($node->isa('RDF::Trine::Node::Blank')) {
+	} elsif ($node->is_blank) {
 		$node_label	= $node->blank_identifier;
 		return $name => { type => 'bnode', value => $node_label };
 	} else {
@@ -621,19 +456,17 @@ L<JSON|JSON>
 
 L<Scalar::Util|Scalar::Util>
 
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
 
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
+Gregory Todd Williams  C<< <greg@evilfunhouse.com> >>
 
-=head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+=head1 LICENCE AND COPYRIGHT
+
+Copyright (c) 2007, Gregory Todd Williams C<< <gwilliams@cpan.org> >>. All rights reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself. See L<perlartistic>.
+
 
-=cut
