@@ -1,14 +1,17 @@
 # RDF::Trine::Namespace
+# -------------
+# $Revision $
+# $Date $
 # -----------------------------------------------------------------------------
 
 
 =head1 NAME
 
-RDF::Trine::Namespace - Abbreviated syntax for constructing RDF node objects
+RDF::Trine::Namespace - Abbreviated syntax for constructing RDF node objects.
 
 =head1 VERSION
 
-This document describes RDF::Trine::Namespace version 0.999_02
+This document describes RDF::Trine::Namespace version 0.0.1
 
 =head1 SYNOPSIS
 
@@ -35,21 +38,14 @@ appended to the namespace.
 
 package RDF::Trine::Namespace;
 
+our $VERSION = '0.001';
+
 use strict;
 use warnings;
 no warnings 'redefine';
+
 use base qw(XML::Namespace);
 
-######################################################################
-
-our ($VERSION);
-BEGIN {
-	$VERSION	= '0.999_02';
-}
-
-######################################################################
-
-
 use Carp;
 use RDF::Trine::Node::Resource;
 use XML::CommonNS 0.04 ();
@@ -65,13 +61,13 @@ sub _install_namespaces {
 	my $class	= shift;
 	my $level	= shift;
 	my $pkg		= caller( $level );
-	foreach my $name (@_) {
-		my $uri	= (uc($name) eq 'XSD')
-			? XML::NamespaceFactory->new('http://www.w3.org/2001/XMLSchema#')
-			: XML::CommonNS->uri( uc($name) );
-		my $ns	= $class->new( "$uri" );
-		no strict 'refs';
-		*{ "${pkg}::${name}" }	= \$ns;
+	if (@_) {
+		foreach my $name (@_) {
+			my $uri	= XML::CommonNS->uri( uc($name) );
+			my $ns	= __PACKAGE__->new( "$uri" );
+			no strict 'refs';
+			*{ "${pkg}::${name}" }	= \$ns;
+		}
 	}
 }
 
@@ -88,16 +84,6 @@ sub uri {
 	return RDF::Trine::Node::Resource->new( $uri );
 }
 
-=item C<< uri_value >>
-
-Returns the URI/IRI value of this namespace.
-
-=cut
-
-sub uri_value {
-	my $self	= shift;
-	return $self->uri();
-}
 
 1; # Magic true value required at end of module
 __END__
@@ -108,19 +94,21 @@ __END__
 
 L<XML::Namespace>
 
-=head1 BUGS
+=head1 BUGS AND LIMITATIONS
 
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
+No bugs have been reported.
+
+Please report any bugs or feature requests to
+C<greg@evilfunhouse.com>.
 
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
+Gregory Todd Williams  C<< <greg@evilfunhouse.com> >>
+
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2006-2007 Gregory Todd Williams. All rights reserved. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
-=cut
