@@ -1,14 +1,13 @@
 # RDF::Trine::Node::Blank
+# -------------
+# $Revision: 121 $
+# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
 RDF::Trine::Node::Blank - RDF Node class for blank nodes
 
-=head1 VERSION
-
-This document describes RDF::Trine::Node::Blank version 0.999_02
-
 =cut
 
 package RDF::Trine::Node::Blank;
@@ -24,21 +23,16 @@ use Carp qw(carp croak confess);
 
 ######################################################################
 
-our ($VERSION);
+our ($VERSION, $debug, $lang, $languri);
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug		= 0;
+	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
 }
 
 ######################################################################
 
-use overload	'""'	=> sub { $_[0]->sse },
-			;
-
 =head1 METHODS
 
-Beyond the methods documented below, this class inherits methods from the
-L<RDF::Trine::Node> class.
-
 =over 4
 
 =cut
@@ -56,15 +50,6 @@ sub new {
 	unless (defined($name)) {
 		$name	= 'r' . time() . 'r' . $COUNTER++;
 	}
-	if ($name =~ m/[^A-Za-z0-9]/) {
-		throw RDF::Trine::Error::SerializationError -text => "Only alphanumerics are allowed in N-Triples bnode labels";
-	}
-	return $class->_new( $name );
-}
-
-sub _new {
-	my $class	= shift;
-	my $name	= shift;
 	return bless( [ 'BLANK', $name ], $class );
 }
 
@@ -79,17 +64,6 @@ sub blank_identifier {
 	return $self->[1];
 }
 
-=item C<< value >>
-
-Returns the blank identifier.
-
-=cut
-
-sub value {
-	my $self	= shift;
-	return $self->blank_identifier;
-}
-
 =item C<< sse >>
 
 Returns the SSE string for this blank node.
@@ -102,16 +76,15 @@ sub sse {
 	return qq(_:${id});
 }
 
-=item C<< as_ntriples >>
+=item C<< as_sparql >>
 
-Returns the node in a string form suitable for NTriples serialization.
+Returns the SPARQL string for this node.
 
 =cut
 
-sub as_ntriples {
+sub as_sparql {
 	my $self	= shift;
-	my $id		= $self->blank_identifier;
-	return qq(_:${id});
+	return $self->sse;
 }
 
 =item C<< as_string >>
@@ -144,36 +117,19 @@ Returns true if the two nodes are equal, false otherwise.
 sub equal {
 	my $self	= shift;
 	my $node	= shift;
-	return 0 unless (blessed($node) and $node->isa('RDF::Trine::Node::Blank'));
+	return 0 unless (blessed($node) and $node->isa('RDF::Trine::Node'));
+	return 0 unless ($self->type eq $node->type);
 	return ($self->blank_identifier eq $node->blank_identifier);
 }
 
-# called to compare two nodes of the same type
-sub _compare {
-	my $a	= shift;
-	my $b	= shift;
-	return ($a->blank_identifier cmp $b->blank_identifier);
-}
-
 1;
 
 __END__
 
 =back
 
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
-
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+ Gregory Todd Williams <gwilliams@cpan.org>
 
 =cut
