@@ -1,13 +1,12 @@
 # RDF::Trine::Statement::Quad
+# -------------
+# $Revision: 121 $
+# $Date: 2006-02-06 23:07:43 -0500 (Mon, 06 Feb 2006) $
 # -----------------------------------------------------------------------------
 
 =head1 NAME
 
-RDF::Trine::Statement::Quad - Class for quads and quad patterns
-
-=head1 VERSION
-
-This document describes RDF::Trine::Statement::Quad version 0.999_02
+RDF::Trine::Statement::Quad - Algebra class for Triple patterns
 
 =cut
 
@@ -18,23 +17,18 @@ use warnings;
 no warnings 'redefine';
 use base qw(RDF::Trine::Statement);
 
-use Scalar::Util qw(blessed);
-use Carp qw(croak);
-
 ######################################################################
 
-our ($VERSION);
+our ($VERSION, $debug);
 BEGIN {
-	$VERSION	= '0.999_02';
+	$debug		= 0;
+	$VERSION	= do { my $REV = (qw$Revision: 121 $)[1]; sprintf("%0.3f", 1 + ($REV/1000)) };
 }
 
 ######################################################################
 
 =head1 METHODS
 
-Beyond the methods documented below, this class inherits methods from the
-L<RDF::Trine::Statement> class.
-
 =over 4
 
 =cut
@@ -48,11 +42,9 @@ Returns a new Quad structure.
 sub new {
 	my $class	= shift;
 	my @nodes	= @_;
-	unless (scalar(@nodes) == 4) {
-		throw RDF::Trine::Error::MethodInvocationError -text => "Quad constructor must have four node arguments";
-	}
+	Carp::confess "Quad constructor must have four node arguments" unless (scalar(@nodes) == 4);
 	my @names	= qw(subject predicate object context);
-	foreach my $i (0 .. 3) {
+	foreach my $i (0 .. 2) {
 		unless (defined($nodes[ $i ])) {
 			$nodes[ $i ]	= RDF::Trine::Node::Variable->new($names[ $i ]);
 		}
@@ -69,17 +61,11 @@ Returns the subject, predicate and object of the triple pattern.
 
 sub nodes {
 	my $self	= shift;
-	return @$self;
-}
-
-=item C<< node_names >>
-
-Returns the method names for accessing the nodes of this statement.
-
-=cut
-
-sub node_names {
-	return qw(subject predicate object context);
+	my $s		= $self->subject;
+	my $p		= $self->predicate;
+	my $o		= $self->object;
+	my $c		= $self->context;
+	return ($s, $p, $o, $c);
 }
 
 =item C<< context >>
@@ -98,17 +84,20 @@ sub context {
 
 =item C<< sse >>
 
-Returns the SSE string for this algebra expression.
+Returns the SSE string for this alegbra expression.
 
 =cut
 
 sub sse {
 	my $self	= shift;
 	my $context	= shift;
-	
-	my @nodes	= $self->nodes;
-	my @sse		= map { $_->sse( $context ) } (@nodes);
-	return sprintf( '(quad %s %s %s %s)', @sse );
+	return sprintf(
+		'(quad %s %s %s %s)',
+		$self->subject->sse( $context ),
+		$self->predicate->sse( $context ),
+		$self->object->sse( $context ),
+		$self->context->sse( $context ),
+	);
 }
 
 =item C<< type >>
@@ -162,7 +151,7 @@ sub from_redland {
 						: undef;
 			return RDF::Trine::Node::Literal->new( $node->literal_value, $lang, $dt );
 		} else {
-			croak 'Unknown node type in statement conversion';
+			die;
 		}
 	};
 	
@@ -183,19 +172,8 @@ __END__
 
 =back
 
-=head1 BUGS
-
-Please report any bugs or feature requests to through the GitHub web interface
-at L<https://github.com/kasei/perlrdf/issues>.
-
 =head1 AUTHOR
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2006-2012 Gregory Todd Williams. This
-program is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+ Gregory Todd Williams <gwilliams@cpan.org>
 
 =cut
